/*
 *   ORXONOX - the hottest 3D action shooter ever to exist
 *                    > www.orxonox.net <
 *
 *
 *   License notice:
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License
 *   as published by the Free Software Foundation; either version 2
 *   of the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 *
 *   Author:
 *      Fabian 'x3n' Landau
 *   Co-authors:
 *      Dominik Solenicki
 *
 */

#include "WingmanController.h"


namespace orxonox
{

    RegisterClass(WingmanController);
    

    WingmanController::WingmanController(Context* context) : CommonController(context)
    {
        RegisterObject(WingmanController);
        this->actionTimer_.setTimer(ACTION_INTERVAL, true, createExecutor(createFunctor(&WingmanController::action, this)));
        this->myLeader_ = 0;
        this->rank_ = WINGMAN;

    }

    WingmanController::~WingmanController()
    {

    }

    void WingmanController::tick(float dt)
    {   
        //-------------------------------------------------------
           
        
        if (!this->isActive())
            return;
        //--------------------------Stay in formation--------------------------
        if (this->bHasTargetPosition_)
        {
            //targetPosition_ and targetOrientation_ are set by the Leader in its action()
            this->moveToTargetPosition();
        } 
        
        //--------------------------Attack same target as the Leader--------------------------

        /*if (this->target_)
        {
            this->aimAtTarget();
            this->doFire();
        }
        */
        
        //orxout(internal_error) << "I am " << this << endl;

        
        SUPER(WingmanController, tick, dt);
    }
    
    void WingmanController::action()
    {
        if (!this->myLeader_)
        {
            CommonController* newLeader = findNewLeader();
            this->myLeader_ = newLeader;
            if (newLeader)
                orxout(internal_error) << "new Leader set" << endl;
            else
                orxout(internal_error) << "0 leader" << endl;

        }
        else
        {

        }
    }
     
   
    

    CommonController* WingmanController::findNewLeader()
    {

        if (!this->getControllableEntity())
            return 0;

        CommonController* closestLeader = 0;
        float minDistance =  std::numeric_limits<float>::infinity();

        for (ObjectList<CommonController>::iterator it = ObjectList<CommonController>::begin(); it; ++it)
        {
            //0ptr?
            if (!it || 
                (it->getRank() != SECTIONLEADER && it->getRank() != DIVISIONLEADER) || 
                !(it->getControllableEntity()))
                continue;
            //same team?
            if (this->getControllableEntity()->getTeam() != (it)->getControllableEntity()->getTeam())
                continue;
            //is equal to this?
            if (it->getControllableEntity() == this->getControllableEntity())
                continue;

            float distance = (it->getControllableEntity()->getPosition() - this->getControllableEntity()->getPosition()).length();
            if (distance < minDistance && !(it->hasWingman()))
            {
                closestLeader = *it;
                minDistance = distance;
            }
           
        }
        if (closestLeader)
        {
            if (closestLeader->setWingman(this))
                return closestLeader;
        }
        return 0;
    }

    void WingmanController::XMLPort(Element& xmlelement, XMLPort::Mode mode)
    {
        SUPER(WingmanController, XMLPort, xmlelement, mode);

        //XMLPortParam(SectionController, "target_", setTarget, getTarget, xmlelement, mode).defaultValues(100.0f);
    }



}
